@@ -28,6 +28,7 @@
 #include "colourgui.h"
 #include "colourscheme.h"
 #include "configlet.h"
+#include "dlg.h"
 #include "dynopts.h"
 
 #define CG_CONNECT(cg, h) \
@@ -40,49 +41,51 @@ struct ColourGUI {
 	GtkWidget *widget;
 	char *scheme_name;
 	Options *colour_scheme;
+	Options *orig_scheme;
 	gboolean ignore_destroy;
 };
 
 static GHashTable *colourgui_being_edited = NULL;
 
 static void colourgui_set_colour_widget(GladeXML *glade,
-		const char *widget_name, GdkColor *colour)
+		const char *widget_name, GdkColor *colour, gboolean ignore)
 {
 	static GdkColormap *colour_map = NULL;
+	GtkWidget *widget = glade_xml_get_widget(glade, widget_name);
 
 	if (!colour_map)
 		colour_map = gdk_colormap_get_system();
 	g_return_if_fail(gdk_colormap_alloc_color(colour_map, colour,
 				TRUE, TRUE));
+	/* gtk_color_button_set_color doesn't seem to cause a change signal so
+	 * explicitly raise one if necessary */
 	capplet_ignore_changes = TRUE;
 	gtk_color_button_set_color(
-			GTK_COLOR_BUTTON(glade_xml_get_widget(glade, widget_name)),
+			GTK_COLOR_BUTTON(widget),
 			colour);
 	capplet_ignore_changes = FALSE;
+	if (!ignore)
+		g_signal_emit_by_name(widget, "color-set");
 }
 
 static void colourgui_set_fgbg_colour_widgets(GladeXML *glade,
-		Options *colour_scheme)
+		Options *colour_scheme, gboolean ignore)
 {
-	capplet_ignore_changes = TRUE;
 	colourgui_set_colour_widget(glade, "foreground_colour",
-			colour_scheme_get_foreground_colour(colour_scheme, TRUE));
+			colour_scheme_get_foreground_colour(colour_scheme, TRUE), ignore);
 	colourgui_set_colour_widget(glade, "background_colour",
-			colour_scheme_get_background_colour(colour_scheme, TRUE));
-	capplet_ignore_changes = FALSE;
+			colour_scheme_get_background_colour(colour_scheme, TRUE), ignore);
 }
 
 static void colourgui_set_cursor_colour_widget(GladeXML *glade,
-		Options *colour_scheme)
+		Options *colour_scheme, gboolean ignore)
 {
-	capplet_ignore_changes = TRUE;
 	colourgui_set_colour_widget(glade, "cursor_colour",
-			colour_scheme_get_cursor_colour(colour_scheme, TRUE));
-	capplet_ignore_changes = FALSE;
+			colour_scheme_get_cursor_colour(colour_scheme, TRUE), ignore);
 }
 
 static void colourgui_set_all_palette_widgets(GladeXML *glade,
-		Options *colour_scheme)
+		Options *colour_scheme, gboolean ignore)
 {
 	int n;
 	GdkColor *palette = colour_scheme_get_palette(colour_scheme);
@@ -93,7 +96,7 @@ static void colourgui_set_all_palette_widgets(GladeXML *glade,
 		char widget_name[20];
 
 		sprintf(widget_name, "palette%d_%d", n / 8, n % 8);
-		colourgui_set_colour_widget(glade, widget_name, &palette[n]);
+		colourgui_set_colour_widget(glade, widget_name, &palette[n], ignore);
 	}
 }
 
@@ -165,12 +168,13 @@ static void colourgui_set_cursor_shading(GladeXML *glade)
 					glade_xml_get_widget(glade, "set_cursor_colour"))));
 }
 
-static void colourgui_set_fgbg_toggle(GladeXML *glade, Options *colour_scheme)
+static void colourgui_set_fgbg_toggle(GladeXML *glade, Options *colour_scheme,
+		gboolean ignore)
 {
 	char *fg = options_lookup_string(colour_scheme, "foreground");
 	char *bg = options_lookup_string(colour_scheme, "background");
 
-	capplet_ignore_changes = TRUE;
+	capplet_ignore_changes = ignore;
 	gtk_toggle_button_set_active(
 			GTK_TOGGLE_BUTTON(glade_xml_get_widget(glade, "set_fgbg")),
 			fg || bg);
@@ -182,7 +186,7 @@ static void colourgui_set_fgbg_toggle(GladeXML *glade, Options *colour_scheme)
 }
 
 static void colourgui_set_fgbg_track_toggle(GladeXML *glade,
-		Options *colour_scheme)
+		Options *colour_scheme, gboolean ignore)
 {
 	gboolean track = FALSE;
 
@@ -204,7 +208,7 @@ static void colourgui_set_fgbg_track_toggle(GladeXML *glade,
 			track = TRUE;
 		}
 	}
-	capplet_ignore_changes = TRUE;
+	capplet_ignore_changes = ignore;
 	gtk_toggle_button_set_active(
 		GTK_TOGGLE_BUTTON(glade_xml_get_widget(glade, "fgbg_track_palette")),
 		track);
@@ -212,13 +216,13 @@ static void colourgui_set_fgbg_track_toggle(GladeXML *glade,
 }
 
 static void colourgui_set_palette_toggles(GladeXML *glade,
-		Options *colour_scheme)
+		Options *colour_scheme, gboolean ignore)
 {
 	int ps = colour_scheme_get_palette_size(colour_scheme); 
 	GtkToggleButton *ucp =
 		GTK_TOGGLE_BUTTON(glade_xml_get_widget(glade, "use_custom_colours"));
 
-	capplet_ignore_changes = TRUE;
+	capplet_ignore_changes = ignore;
 	switch (ps)
 	{
 		case 0:
@@ -243,11 +247,11 @@ static void colourgui_set_palette_toggles(GladeXML *glade,
 }
 
 static void colourgui_set_cursor_toggle(GladeXML *glade,
-		Options *colour_scheme)
+		Options *colour_scheme, gboolean ignore)
 {
 	char *cc = options_lookup_string(colour_scheme, "cursor");
 
-	capplet_ignore_changes = TRUE;
+	capplet_ignore_changes = ignore;
 	gtk_toggle_button_set_active(
 			GTK_TOGGLE_BUTTON(glade_xml_get_widget(glade, "set_cursor_colour")),
 			 cc != NULL);
@@ -255,12 +259,13 @@ static void colourgui_set_cursor_toggle(GladeXML *glade,
 	g_free(cc);
 }
 
-static void colourgui_set_all_toggles(GladeXML *glade, Options *colour_scheme)
+static void colourgui_set_all_toggles(GladeXML *glade, Options *colour_scheme,
+		gboolean ignore)
 {
-	colourgui_set_fgbg_toggle(glade, colour_scheme);
-	colourgui_set_fgbg_track_toggle(glade, colour_scheme);
-	colourgui_set_palette_toggles(glade, colour_scheme);
-	colourgui_set_cursor_toggle(glade, colour_scheme);
+	colourgui_set_fgbg_toggle(glade, colour_scheme, ignore);
+	colourgui_set_fgbg_track_toggle(glade, colour_scheme, ignore);
+	colourgui_set_palette_toggles(glade, colour_scheme, ignore);
+	colourgui_set_cursor_toggle(glade, colour_scheme, ignore);
 }
 
 static void colourgui_set_all_shading(GladeXML *glade)
@@ -271,6 +276,25 @@ static void colourgui_set_all_shading(GladeXML *glade)
 	colourgui_set_cursor_shading(glade);
 }
 
+static void colourgui_fill_in_dialog(ColourGUI * cg, gboolean ignore)
+{
+	GladeXML *glade = cg->glade;
+	Options *colour_scheme = cg->colour_scheme;
+
+	colourgui_set_all_toggles(glade, colour_scheme, ignore);
+	colourgui_set_all_shading(glade);
+	colourgui_set_all_palette_widgets(glade, colour_scheme, ignore);
+	colourgui_set_fgbg_colour_widgets(glade, colour_scheme, ignore);
+	colourgui_set_cursor_colour_widget(glade, colour_scheme, ignore);
+}
+
+static void colourgui_revert(ColourGUI *cg)
+{
+	options_copy_keyfile(cg->colour_scheme, cg->orig_scheme);
+	colour_scheme_reset_cached_data(cg->colour_scheme);
+	colourgui_fill_in_dialog(cg, FALSE);
+}
+
 /***********************************************************/
 /* Handlers */
 
@@ -290,7 +314,10 @@ static void on_Colour_Editor_destroy(GtkWidget *widget, ColourGUI * cg)
 static void on_Colour_Editor_response(GtkWidget *widget, gint response,
 		ColourGUI * cg)
 {
-	colourgui_delete(cg);
+	if (response == 1)
+		colourgui_revert(cg);
+	else
+		colourgui_delete(cg);
 }
 
 static void on_Colour_Editor_close(GtkWidget *widget, ColourGUI * cg)
@@ -390,14 +417,14 @@ static void on_color_set(GtkColorButton *button, ColourGUI * cg)
 			if (column == 7)
 			{
 				colourgui_set_colour_widget(cg->glade,
-						"background_colour", &colour);
+						"background_colour", &colour, TRUE);
 				capplet_set_string(cg->colour_scheme,
 						"background", colour_name);
 			}
 			if (column == 0)
 			{
 				colourgui_set_colour_widget(cg->glade,
-						"foreground_colour", &colour);
+						"foreground_colour", &colour, TRUE);
 				capplet_set_string(cg->colour_scheme,
 						"foreground", colour_name);
 			}
@@ -422,7 +449,8 @@ static void on_set_fgbg_toggled(GtkToggleButton *button, ColourGUI * cg)
 		return;
 
 	colourgui_set_fgbg_shading(cg->glade);
-	colourgui_set_fgbg_track_toggle(cg->glade, cg->colour_scheme);
+	colourgui_set_fgbg_track_shading(cg->glade);
+	colourgui_set_fgbg_track_toggle(cg->glade, cg->colour_scheme, FALSE);
 	state = gtk_toggle_button_get_active(button);
 	if (state)
 	{
@@ -446,7 +474,7 @@ static void mirror_colour(ColourGUI *cg, const char *source_widget_name,
 		GTK_COLOR_BUTTON(glade_xml_get_widget(cg->glade, source_widget_name)),
 		&colour);
 	colour_name = colourgui_name_colour(&colour);
-	colourgui_set_colour_widget(cg->glade, target_widget, &colour);
+	colourgui_set_colour_widget(cg->glade, target_widget, &colour, TRUE);
 	capplet_set_string(cg->colour_scheme, target_option, colour_name);
 }
 
@@ -499,11 +527,9 @@ static void on_use_custom_colours_toggled(GtkToggleButton *button,
 		colour_scheme_set_palette_size(cg->colour_scheme, palette_size);
 		if (!palette_size)
 		{
-			capplet_ignore_changes = TRUE;
 			gtk_toggle_button_set_active(
 						GTK_TOGGLE_BUTTON(glade_xml_get_widget(cg->glade,
 								"fgbg_track_palette")), FALSE);
-			capplet_ignore_changes = FALSE;
 		}
 		colourgui_set_fgbg_track_shading(cg->glade);
 		colourgui_set_palette_shading(cg->glade);
@@ -580,18 +606,6 @@ static void colourgui_connect_handlers(ColourGUI * cg)
 	CG_CONNECT(cg, on_use_custom_colours_toggled);
 }
 
-static void colourgui_fill_in_dialog(ColourGUI * cg)
-{
-	GladeXML *glade = cg->glade;
-	Options *colour_scheme = cg->colour_scheme;
-
-	colourgui_set_all_toggles(glade, colour_scheme);
-	colourgui_set_all_shading(glade);
-	colourgui_set_all_palette_widgets(glade, colour_scheme);
-	colourgui_set_fgbg_colour_widgets(glade, colour_scheme);
-	colourgui_set_cursor_colour_widget(glade, colour_scheme);
-}
-
 ColourGUI *colourgui_open(const char *scheme_name)
 {
 	ColourGUI *cg;
@@ -612,15 +626,23 @@ ColourGUI *colourgui_open(const char *scheme_name)
 	cg = g_new0(ColourGUI, 1);
 	cg->scheme_name = g_strdup(scheme_name);
 	cg->colour_scheme = colour_scheme_lookup_and_ref(scheme_name);
+	cg->orig_scheme = options_copy(cg->colour_scheme);
 
 	cg->glade = glade_xml_new(capplet_get_glade_filename(),
 		"Colour_Scheme_Editor", NULL);
 	g_assert(cg->glade);
 	cg->widget = glade_xml_get_widget(cg->glade, "Colour_Scheme_Editor");
+	if (!cg->orig_scheme)
+	{
+		dlg_warning(GTK_WINDOW(cg->widget),
+				_("Unable to back up colour scheme to revert"));
+		gtk_widget_set_sensitive(glade_xml_get_widget(cg->glade, "revert"),
+				FALSE);
+	}
 
 	g_hash_table_insert(colourgui_being_edited, g_strdup(scheme_name), cg);
 
-	colourgui_fill_in_dialog(cg);
+	colourgui_fill_in_dialog(cg, TRUE);
 	colourgui_connect_handlers(cg);
 
 	gtk_widget_show(cg->widget);
@@ -646,6 +668,8 @@ void colourgui_delete(ColourGUI * cg)
 	}
 	UNREF_LOG(g_object_unref(cg->glade));
 	g_free(cg->scheme_name);
+	if (cg->orig_scheme)
+		options_delete(cg->orig_scheme);
 	UNREF_LOG(colour_scheme_unref(cg->colour_scheme));
 	g_free(cg);
 	capplet_dec_windows();
