@@ -246,7 +246,7 @@ void global_options_init_appdir(int argc, char **argv)
 	}
 }
 
-static void global_options_parse_argv(int argc, char **argv)
+static void global_options_parse_argv(int *argc, char ***argv)
 {
 	GOptionContext *octx = g_option_context_new(NULL);
 	GOptionGroup *ogroup = gtk_get_option_group(FALSE);
@@ -255,7 +255,7 @@ static void global_options_parse_argv(int argc, char **argv)
 	g_option_context_set_help_enabled(octx, TRUE);
 	g_option_context_add_group(octx, ogroup);
 	g_option_context_add_main_entries(octx, global_g_options, NULL);
-	g_option_context_parse(octx, &argc, &argv, &err);
+	g_option_context_parse(octx, argc, argv, &err);
 	if (err)
 	{
 		dlg_warning(NULL, _("Error parsing command line options: %s"),
@@ -264,7 +264,7 @@ static void global_options_parse_argv(int argc, char **argv)
 	}
 }
 
-void global_options_init(int argc, char **argv)
+void global_options_init(int *argc, char ***argv)
 {
 	static gboolean already = FALSE;
 
@@ -274,7 +274,7 @@ void global_options_init(int argc, char **argv)
 	}
 	else
 	{
-		global_options_init_appdir(argc, argv);
+		global_options_init_appdir(*argc, *argv);
 		/* Need to get appdir before trying to load any options files */
 		global_options = options_open("Global", "roxterm options");
 		correct_schemes();
@@ -282,7 +282,7 @@ void global_options_init(int argc, char **argv)
 	global_options_parse_argv(argc, argv);
 	if (!already)
 	{
-		global_options_bindir = g_path_get_dirname(argv[0]);
+		global_options_bindir = g_path_get_dirname((*argv)[0]);
 		if (!strcmp(global_options_bindir, "."))
 		{
 			/* No directory component, try to find roxterm-config in BIN_DIR,
