@@ -25,7 +25,8 @@
 /* Call before gtk_init parses argv in case there's an execute option which may
  * get corrupted by gtk_init; returns FALSE if there's a -e/--execute with no
  * subsequent args; use shallow_copy if you want original strings after -e
- * to be freed later */
+ * to be freed later; argc is reduced to exclude any args after and including
+ * -e/--execute, but argv isn't actually changed */
 gboolean global_options_preparse_argv_for_execute(int *argc, char **argv,
 		gboolean shallow_copy);
 
@@ -47,7 +48,7 @@ extern gboolean global_options_fullscreen;
 /* Call after argv has been processed by gtk_init; may be called more than once
  * but repeat invocations have no effect on appdir/bindir. Bear in mind that
  * --help/--usage args will cause exit. */
-void global_options_init(int argc, char **argv);
+void global_options_init(int *argc, char ***argv);
 
 /* Only access via following functions */
 extern Options *global_options;
@@ -74,7 +75,7 @@ inline static double global_options_lookup_double(const char *key)
 	return options_lookup_double(global_options, key);
 }
 
-/* Checks CLI args for --appdir */
+/* Checks CLI args for --appdir without altering argv */
 void global_options_init_appdir(int argc, char **argv);
 
 /* Deep copy a NULL-terminated array of NULL-terminated strings */
