@@ -431,9 +431,9 @@ static void menutree_build(MenuTree *menu_tree, Options *shortcuts,
 	{
 		menutree_build_shell(menu_tree, GTK_MENU_SHELL(menu_tree->top_level),
 			disable_shortcuts, 
-			_("Open URL in browser"), MENUTREE_OPEN_IN_BROWSER,
-			_("Send email"), MENUTREE_OPEN_IN_MAILER,
-			_("Copy address to clipboard"), MENUTREE_COPY_URI,
+			_("_Open URL in browser"), MENUTREE_OPEN_IN_BROWSER,
+			_("_Send email"), MENUTREE_OPEN_IN_MAILER,
+			_("_Copy address to clipboard"), MENUTREE_COPY_URI,
 			_("_"), MENUTREE_URI_SEPARATOR, TOP_LEVEL_MENU_ITEMS);
 	}
 
@@ -453,6 +453,7 @@ static void menutree_build(MenuTree *menu_tree, Options *shortcuts,
 		_("_Copy"), MENUTREE_EDIT_COPY,
 		_("_Paste"), MENUTREE_EDIT_PASTE,
 		"_", MENUTREE_NULL_ID,
+		_("_Set Window Title..."), MENUTREE_EDIT_SET_WINDOW_TITLE,
 		_("_Reset"), MENUTREE_EDIT_RESET,
 		_("Reset And C_lear"), MENUTREE_EDIT_RESET_AND_CLEAR, NULL);
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_tree->item_widgets
