@@ -80,6 +80,8 @@ struct MultiWin {
 	gboolean wrap_switch_tab;
 	int zoom_index;
 	gboolean fullscreen;
+	char *title_template;
+	char *child_title;
 };
 
 static double multi_win_zoom_factors[] = {
@@ -127,6 +129,8 @@ static void multi_win_select_tab_action(GtkCheckMenuItem *widget,
 
 static void multi_win_destructor(MultiWin *win, gboolean destroy_widgets);
 
+static void multi_win_set_title(MultiWin *win, const char *title);
+
 inline static MultiTab *multi_tab_from_widget(GtkWidget *widget)
 {
 	return g_object_get_data(G_OBJECT(widget), "roxterm_tab");
@@ -268,8 +272,7 @@ void multi_tab_set_title(MultiTab * tab, const char *title)
 	tab->title = title ? g_strdup(title) : NULL;
 	if (win->current_tab == tab)
 	{
-		gtk_window_set_title(GTK_WINDOW(win->gtkwin),
-			STR_EMPTY(tab->title));
+		multi_win_set_title(win, STR_EMPTY(tab->title));
 	}
 	if (!tab->name)
 		multi_tab_set_name_or_title(tab, title);
@@ -556,6 +559,31 @@ static void multi_win_set_geometry_hints_for_tab(MultiWin * win, MultiTab * tab)
 	}
 }
 
+static void multi_win_set_title(MultiWin *win, const char *title)
+{
+	char *title0 = NULL;
+	
+	if (win->child_title != title)
+	{
+		g_free(win->child_title);
+		win->child_title = title ? g_strdup(title) : NULL;
+	}
+	if (win->title_template)
+	{
+		if (strstr(win->title_template, "%s"))
+		{
+			title0 = g_strdup_printf(win->title_template, title);
+			title = title0;
+		}
+		else
+		{
+			title = win->title_template;
+		}
+	}
+	gtk_window_set_title(GTK_WINDOW(win->gtkwin), title);
+	g_free(title0);
+}
+
 void multi_win_select_tab(MultiWin * win, MultiTab * tab)
 {
 	/* This is called by anything that causes a change of tab to keep things
@@ -572,7 +600,7 @@ void multi_win_select_tab(MultiWin * win, MultiTab * tab)
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(win->notebook),
 			multi_tab_get_page_num(tab));
 		gtk_widget_grab_focus(tab->active_widget);
-		gtk_window_set_title(GTK_WINDOW(win->gtkwin), tab->title);
+		multi_win_set_title(win, tab->title);
 		menutree_select_tab(win->popup_menu, tab->popup_menu_item);
 		menutree_select_tab(win->menu_bar, tab->menu_bar_item);
 		if (GTK_WIDGET_REALIZED(tab->active_widget) &&
@@ -1608,6 +1636,53 @@ GtkPositionType multi_win_get_tab_pos(MultiWin *win)
 	return win->tab_pos;
 }
 
+void multi_win_set_title_template(MultiWin *win, const char *tt)
+{
+	g_free(win->title_template);
+	win->title_template = tt ? g_strdup(tt) : NULL;
+	if (win->child_title)
+		multi_win_set_title(win, win->child_title);
+}
+
+gboolean multi_win_validate_title_template(const char *tt)
+{
+	const char *c;
+	gboolean lwpc = FALSE;
+	gboolean got_pcs = FALSE;
+	
+	c = tt;
+	do
+	{
+		if (*c == '%')
+		{
+			/* Check for illegal trailing % */
+			if (!lwpc && *(c + 1) == 0)
+				return FALSE;
+			/* lwpc is FALSE after even # of %s, TRUE after odd # */
+			lwpc = !lwpc;
+		}
+		else
+		{
+			if (lwpc)
+			{
+				if (*c == 's')
+				{
+					if (got_pcs)
+						return FALSE;
+					got_pcs = TRUE;
+				}
+				else
+				{
+					return FALSE;
+				}
+			}
+			lwpc = FALSE;
+		}
+	}
+	while (*(c++));
+	return TRUE;
+}
+
 GList *multi_win_all = NULL;
 
 /* vi:set sw=4 ts=4 noet cindent cino= */
