@@ -292,6 +292,13 @@ GtkPositionType multi_win_get_tab_pos(MultiWin *win);
 /* List of all known windows - treat as read-only */
 extern GList *multi_win_all;
 
+/* Sets a title template string - see profile dialog "Window/Tabs Window Title"
+   comment for details; NULL is equivalent to "%s" */
+void multi_win_set_title_template(MultiWin *win, const char *tt);
+
+/* Returns FALSE if there are any illegal % sequences in tt */
+gboolean multi_win_validate_title_template(const char *tt);
+
 #endif /* MULTITAB_H */
 
 /* vi:set sw=4 ts=4 noet cindent cino= */
