@@ -54,6 +54,7 @@ struct ProfileGUI {
 		char browser;
 		char mailer;
 		char command;
+		char title_string;
 	} changed;
 	GtkWidget *bgimg;
 	DragReceiveData *bgimg_drd;
@@ -95,6 +96,11 @@ void profilegui_check_entries_for_changes(ProfileGUI * pg)
 		profilegui_update_from_entry(pg, "command");
 		pg->changed.command = 0;
 	}
+	if (pg->changed.title_string)
+	{
+		profilegui_update_from_entry(pg, "title_string");
+		pg->changed.command = 0;
+	}
 }
 
 static void profilegui_set_bgimg_shading(ProfileGUI *pg, gboolean sensitive)
@@ -407,6 +413,7 @@ static void profilegui_connect_handlers(ProfileGUI * pg)
 	PG_EC_CONNECT(pg, command);
 	PG_EC_CONNECT(pg, browser);
 	PG_EC_CONNECT(pg, mailer);
+	PG_EC_CONNECT(pg, title_string);
 	/*
     glade_xml_signal_connect_data(pg->glade, "on_browser_changed",
             (GCallback) pg_special_editable_changed, &pg->changed.browser);
@@ -488,6 +495,7 @@ static void profilegui_fill_in_dialog(ProfileGUI * pg)
 	capplet_set_combo(glade, profile, "exit_action", 0);
 	capplet_set_combo(glade, profile, "tab_pos", 0);
 	capplet_set_spin_button_float(glade, profile, "exit_pause");
+	capplet_set_text_entry(glade, profile, "title_string", "%s");
 	profilegui_set_command_shading(pg);
 }
 
