@@ -729,7 +729,8 @@ roxterm_update_cursor_colour(ROXTermData * roxterm, VteTerminal * vte,
 
 guint16 extrapolate_chroma(guint16 bg, guint16 fg, double factor)
 {
-	guint ext = (guint) bg + (guint) ((double) (fg - bg) * factor);
+	gint32 ext = (gint32) bg +
+		(gint32) ((double) ((gint32) fg - (gint32) bg) * factor);
 
 	return (guint16) CLAMP(ext, 0, 0xffff);
 }
